@@ -1,14 +1,17 @@
 from collections import namedtuple
+import datetime as dt
 import logging
 from typing import List, Iterable, Tuple
 
-from django.db import models
+from django.db import models, transaction
 from django.db.utils import IntegrityError
+from django.utils.timezone import now
 
 from bravado.exception import HTTPNotFound
 
 from . import __title__
-from .helpers import EveEntityNameResolver
+from .app_settings import EVEUNIVERSE_BULK_METHODS_BATCH_SIZE
+from .helpers import EveEntityNameResolver, get_or_create_esi_or_none
 from .providers import esi
 from .utils import chunks, LoggerAddTag, make_logger_prefix
 
@@ -46,16 +49,16 @@ class EveUniverseBaseModelManager(models.Manager):
                         try:
                             value = ParentClass.objects.get(id=esi_value)
                         except ParentClass.DoesNotExist:
-                            if mapping.create_related and hasattr(
-                                ParentClass.objects, "update_or_create_esi"
-                            ):
-                                value, _ = ParentClass.objects.update_or_create_esi(
-                                    id=esi_value,
-                                    include_children=False,
-                                    wait_for_children=True,
-                                )
-                            else:
-                                value = None
+                            value = None
+                            if mapping.create_related:
+                                try:
+                                    value, _ = ParentClass.objects.update_or_create_esi(
+                                        id=esi_value,
+                                        include_children=False,
+                                        wait_for_children=True,
+                                    )
+                                except AttributeError:
+                                    pass
 
                     else:
                         if mapping.is_charfield and esi_value is None:
@@ -108,6 +111,7 @@ class EveUniverseEntityModelManager(EveUniverseBaseModelManager):
         id: int,
         include_children: bool = False,
         wait_for_children: bool = True,
+        enabled_sections: Iterable[str] = None,
     ) -> Tuple[models.Model, bool]:
         """updates or creates an Eve universe object by fetching it from ESI (blocking).
         Will always get/create parent objects
@@ -116,11 +120,13 @@ class EveUniverseEntityModelManager(EveUniverseBaseModelManager):
             id: Eve Online ID of object
             include_children: if child objects should be updated/created as well (if any)
             wait_for_children: when true child objects will be updated/created blocking (if any), else async
+            enabled_sections: Sections to load regardless of current settings, e.g. `EveUniverseEntityModel.LOAD_DOGMAS` will always load dogmas for EveTypes
 
         Returns:
             A tuple consisting of the requested object and a created flag
         """
         id = int(id)
+        enabled_sections = set(enabled_sections) if enabled_sections else set()
         add_prefix = make_logger_prefix("%s(id=%s)" % (self.model.__name__, id))
         try:
             eve_data_obj = self._transform_esi_response_for_list_endpoints(
@@ -129,18 +135,20 @@ class EveUniverseEntityModelManager(EveUniverseBaseModelManager):
             if eve_data_obj:
                 defaults = self._defaults_from_esi_obj(eve_data_obj)
                 obj, created = self.update_or_create(id=id, defaults=defaults)
-                inline_objects = self.model._inline_objects()
+                inline_objects = self.model._inline_objects(enabled_sections)
                 if inline_objects:
                     self._update_or_create_inline_objects(
                         parent_eve_data_obj=eve_data_obj,
                         parent_obj=obj,
                         inline_objects=inline_objects,
+                        wait_for_children=wait_for_children,
                     )
-                if eve_data_obj and include_children:
+                if include_children:
                     self._update_or_create_children(
                         parent_eve_data_obj=eve_data_obj,
                         include_children=include_children,
                         wait_for_children=wait_for_children,
+                        enabled_sections=enabled_sections,
                     )
             else:
                 raise HTTPNotFound(
@@ -196,10 +204,15 @@ class EveUniverseEntityModelManager(EveUniverseBaseModelManager):
         parent_eve_data_obj: dict,
         parent_obj: models.Model,
         inline_objects: dict,
+        wait_for_children: bool,
     ) -> None:
         """updates_or_creates eve objects that are returns "inline" from ESI
         for the parent eve objects as defined for this parent model (if any)
         """
+        from .tasks import (
+            update_or_create_inline_object as task_update_or_create_inline_object,
+        )
+
         if not parent_eve_data_obj or not parent_obj:
             raise ValueError(
                 "%s: Tried to create inline object from empty parent object"
@@ -234,28 +247,66 @@ class EveUniverseEntityModelManager(EveUniverseBaseModelManager):
                         )
                     )
 
+                parent2_model_name = ParentClass2.__name__ if ParentClass2 else None
+                other_pk_info = {
+                    "name": other_pk[0],
+                    "esi_name": other_pk[1].esi_name,
+                    "is_fk": other_pk[1].is_fk,
+                }
                 for eve_data_obj in parent_eve_data_obj[inline_field]:
-                    args = {parent_fk: parent_obj}
-                    esi_value = eve_data_obj[other_pk[1].esi_name]
-                    if other_pk[1].is_fk:
-                        try:
-                            value = ParentClass2.objects.get(id=esi_value)
-                        except ParentClass2.DoesNotExist:
-                            if hasattr(ParentClass2.objects, "update_or_create_esi"):
-                                (
-                                    value,
-                                    _,
-                                ) = ParentClass2.objects.get_or_create_esi(id=esi_value)
-                            else:
-                                value = None
+                    if wait_for_children:
+                        self._update_or_create_inline_object(
+                            parent_obj_id=parent_obj.id,
+                            parent_fk=parent_fk,
+                            eve_data_obj=eve_data_obj,
+                            other_pk_info=other_pk_info,
+                            parent2_model_name=parent2_model_name,
+                            inline_model_name=model_name,
+                        )
                     else:
-                        value = esi_value
+                        task_update_or_create_inline_object(
+                            parent_obj_id=parent_obj.id,
+                            parent_fk=parent_fk,
+                            eve_data_obj=eve_data_obj,
+                            other_pk_info=other_pk_info,
+                            parent2_model_name=parent2_model_name,
+                            inline_model_name=model_name,
+                            parent_model_name=type(parent_obj).__name__,
+                        )
 
-                    args[other_pk[0]] = value
-                    args["defaults"] = InlineModel.objects._defaults_from_esi_obj(
-                        eve_data_obj,
-                    )
-                    InlineModel.objects.update_or_create(**args)
+    def _update_or_create_inline_object(
+        self,
+        parent_obj_id: int,
+        parent_fk: str,
+        eve_data_obj: dict,
+        other_pk_info: dict,
+        parent2_model_name: str,
+        inline_model_name: str,
+    ):
+        """Updates or creates a single inline object.
+        Will automatically create additional parent objects as needed
+        """
+        InlineModel = self.model.get_model_class(inline_model_name)
+
+        args = {f"{parent_fk}_id": parent_obj_id}
+        esi_value = eve_data_obj.get(other_pk_info["esi_name"])
+        if other_pk_info["is_fk"]:
+            ParentClass2 = self.model.get_model_class(parent2_model_name)
+            try:
+                value = ParentClass2.objects.get(id=esi_value)
+            except ParentClass2.DoesNotExist:
+                try:
+                    value, _ = ParentClass2.objects.update_or_create_esi(id=esi_value)
+                except AttributeError:
+                    value = None
+        else:
+            value = esi_value
+
+        args[other_pk_info["name"]] = value
+        args["defaults"] = InlineModel.objects._defaults_from_esi_obj(
+            eve_data_obj,
+        )
+        InlineModel.objects.update_or_create(**args)
 
     def _update_or_create_children(
         self,
@@ -263,9 +314,12 @@ class EveUniverseEntityModelManager(EveUniverseBaseModelManager):
         parent_eve_data_obj: dict,
         include_children: bool,
         wait_for_children: bool,
+        enabled_sections: Iterable[str] = None,
     ) -> None:
         """updates or creates child objects as defined for this parent model (if any)"""
-        from .tasks import update_or_create_eve_object
+        from .tasks import (
+            update_or_create_eve_object as task_update_or_create_eve_object,
+        )
 
         if not parent_eve_data_obj:
             raise ValueError(
@@ -282,14 +336,16 @@ class EveUniverseEntityModelManager(EveUniverseBaseModelManager):
                             id=id,
                             include_children=include_children,
                             wait_for_children=wait_for_children,
+                            enabled_sections=enabled_sections,
                         )
 
                     else:
-                        update_or_create_eve_object.delay(
+                        task_update_or_create_eve_object.delay(
                             child_class,
                             id,
                             include_children=include_children,
                             wait_for_children=wait_for_children,
+                            enabled_sections=list(enabled_sections),
                         )
 
     def update_or_create_all_esi(
@@ -455,6 +511,7 @@ class EveStargateManager(EveUniverseEntityModelManager):
         *,
         include_children: bool = False,
         wait_for_children: bool = True,
+        enabled_sections: Iterable[str] = None,
     ) -> Tuple[models.Model, bool]:
         """updates or creates an EveStargate object by fetching it from ESI (blocking).
         Will always get/create parent objects
@@ -494,6 +551,7 @@ class EveStationManager(EveUniverseEntityModelManager):
         parent_eve_data_obj: dict,
         parent_obj: models.Model,
         inline_objects: dict,
+        wait_for_children: bool,
     ) -> None:
         """updates_or_creates station service objects for EveStations"""
         from .models import EveStationService
@@ -568,6 +626,7 @@ class EveEntityManager(EveUniverseEntityModelManager):
         id: int,
         include_children: bool = False,
         wait_for_children: bool = True,
+        enabled_sections: Iterable[str] = None,
     ) -> Tuple[models.Model, bool]:
         """updates or creates an EveEntity object by fetching it from ESI (blocking).
 
@@ -587,6 +646,7 @@ class EveEntityManager(EveUniverseEntityModelManager):
 
     def bulk_create_esi(self, ids: Iterable[int]) -> int:
         """bulk create and resolve multiple entities from ESI.
+        Will also resolve existing entities, that have no name.
 
         Args:
             ids: List of valid EveEntity IDs
@@ -595,14 +655,18 @@ class EveEntityManager(EveUniverseEntityModelManager):
             Count of updated entities
         """
         ids = set(map(int, ids))
-        existing_ids = set(
-            self.filter(id__in=ids).exclude(name="").values_list("id", flat=True)
-        )
-        new_ids = ids.difference(existing_ids)
-        if new_ids:
-            objects = [self.model(id=id) for id in new_ids]
-            self.bulk_create(objects, ignore_conflicts=True)
-            return self.filter(id__in=new_ids).update_from_esi()
+        with transaction.atomic():
+            existing_ids = set(self.filter(id__in=ids).values_list("id", flat=True))
+            new_ids = ids.difference(existing_ids)
+            if new_ids:
+                objects = [self.model(id=id) for id in new_ids]
+                self.bulk_create(
+                    objects, batch_size=EVEUNIVERSE_BULK_METHODS_BATCH_SIZE
+                )
+                to_update_qs = self.filter(id__in=new_ids) | self.filter(
+                    id__in=ids.difference(new_ids), name=""
+                )
+                return to_update_qs.update_from_esi()
 
         return 0
 
@@ -660,3 +724,55 @@ class EveEntityManager(EveUniverseEntityModelManager):
                 for row in self.filter(id__in=ids).values_list("id", "name")
             }
         )
+
+
+class EveMarketPriceManager(models.Manager):
+    def update_from_esi(self, minutes_until_stale: int = None) -> int:
+        """Updates market prices from ESI. Will only create new price objects for EveTypes that already exist in the database.
+
+        Args:
+            minutes_until_stale: only prices older then given minutes are regarding as stale and will be updated. Will use default (60) if not specified.
+
+        Returns:
+            Count of updated types
+        """
+        from .models import EveType
+
+        if minutes_until_stale is None:
+            minutes_until_stale = self.model.DEFAULT_MINUTES_UNTIL_STALE
+
+        logger.info("Fetching market prices from ESI")
+        entries = esi.client.Market.get_markets_prices().results()
+        if not entries:
+            return 0
+
+        entries_2 = {int(x["type_id"]): x for x in entries if "type_id" in x}
+        with transaction.atomic():
+            existing_types_ids = set(EveType.objects.values_list("id", flat=True))
+            relevant_prices_ids = set(entries_2.keys()).intersection(existing_types_ids)
+            deadline = now() - dt.timedelta(minutes=minutes_until_stale)
+            current_prices_ids = set(
+                self.filter(updated_at__gt=deadline).values_list(
+                    "eve_type_id", flat=True
+                )
+            )
+            need_updating_ids = relevant_prices_ids.difference(current_prices_ids)
+            if not need_updating_ids:
+                logger.info("Market prices are up to date")
+                return 0
+
+            logger.info("Updating market prices for %s types", len(need_updating_ids))
+            self.filter(eve_type_id__in=need_updating_ids).delete()
+            market_prices = [
+                self.model(
+                    eve_type=get_or_create_esi_or_none("type_id", entry, EveType),
+                    adjusted_price=entry.get("adjusted_price"),
+                    average_price=entry.get("average_price"),
+                )
+                for type_id, entry in entries_2.items()
+                if type_id in need_updating_ids
+            ]
+            self.bulk_create(
+                market_prices, batch_size=EVEUNIVERSE_BULK_METHODS_BATCH_SIZE
+            )
+            return len(market_prices)
